# Configure paths for Mino
# Owen Taylor     97-11-3

dnl AM_PATH_MINO([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for MINO, and define MINO_CFLAGS and MINO_LIBS
dnl
AC_DEFUN(AM_PATH_MINO,
[dnl 
dnl Get the cflags and libraries from the mino-config script
dnl
AC_ARG_WITH(mino-prefix,[  --with-mino-prefix=PFX   Prefix where Mino is installed (optional)],
            mino_config_prefix="$withval", mino_config_prefix="")
AC_ARG_WITH(mino-exec-prefix,[  --with-mino-exec-prefix=PFX Exec prefix where Mino is installed (optional)],
            mino_config_exec_prefix="$withval", mino_config_exec_prefix="")
AC_ARG_ENABLE(minotest, [  --disable-minotest       Do not try to compile and run a test Mino program],
            , enable_minotest=yes)


  if test x$mino_config_exec_prefix != x ; then
     mino_config_args="$mino_config_args --exec-prefix=$mino_config_exec_prefix"
     if test x${MINO_CONFIG+set} != xset ; then
        MINO_CONFIG=$mino_config_exec_prefix/bin/mino-config
     fi
  fi
  if test x$mino_config_prefix != x ; then
     mino_config_args="$mino_config_args --prefix=$mino_config_prefix"
     if test x${MINO_CONFIG+set} != xset ; then
        MINO_CONFIG=$mino_config_prefix/bin/mino-config
     fi
  fi

  AC_PATH_PROG(MINO_CONFIG, mino-config, no)
	
  min_mino_version=ifelse([$1], ,0.6.2,$1)
  AC_MSG_CHECKING([for Mino - version >= $min_mino_version])
  no_mino=""
  if test "$MINO_CONFIG" = "no" ; then
    no_mino=yes
  else
		AC_LANG_SAVE
		AC_LANG_CPLUSPLUS

    MINO_CFLAGS=`$MINO_CONFIG $mino_config_args --cflags`
    MINO_LIBS=`$MINO_CONFIG $mino_config_args --libs`
    mino_config_major_version=`$MINO_CONFIG $mino_config_args --version | \
           sed 's/[[^0-9]]*\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    mino_config_minor_version=`$MINO_CONFIG $mino_config_args --version | \
           sed 's/[[^0-9]]*\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    mino_config_micro_version=`$MINO_CONFIG $mino_config_args --version | \
           sed 's/[[^0-9]]*\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    if test "x$enable_minotest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $MINO_CFLAGS"
      LIBS="$MINO_LIBS $LIBS"
dnl
dnl Now check if the installed Mino is sufficiently new. (Also sanity
dnl checks the results of mino-config to some extent
dnl
      rm -f conf.minotest
      AC_TRY_RUN([
#include <mino.h>
#include <stdio.h>
#include <stdlib.h>

char *my_strdup(char *str)
{
	char *new_str;

	if (str)
	{
		new_str = (char *)malloc((strlen(str) + 1) * sizeof(char));
		strcpy(new_str, str);
	}
	else
		new_str = NULL;
	
	return new_str;
}

int 
main ()
{
  int major, minor, micro;
  char *tmp_version;

  system ("touch conf.minotest");

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_mino_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_mino_version");
     exit(1);
   }

  if (($mino_config_major_version  > major) ||
     (($mino_config_major_version == major) && ($mino_config_minor_version > minor)) ||
     (($mino_config_major_version == major) && ($mino_config_minor_version == minor) &&
      ($mino_config_micro_version >= micro)))
    {
			return 0;
		}
		else
		{
      printf("\n*** 'mino-config --version' returned %d.%d.%d, but Mino (%d.%d.%d)\n", 
             $mino_config_major_version, $mino_config_minor_version, $mino_config_micro_version,
             major, minor, micro);
      printf ("*** was found! If mino-config was correct, then it is best\n");
      printf ("*** to remove the old version of Mino. You may also be able to fix the error\n");
      printf("*** by modifying your LD_LIBRARY_PATH enviroment variable, or by editing\n");
      printf("*** /etc/ld.so.conf. Make sure you have run ldconfig if that is\n");
      printf("*** required on your system.\n");
      printf("*** If mino-config was wrong, set the environment variable MINO_CONFIG\n");
      printf("*** to point to the correct copy of mino-config, and remove the file config.cache\n");
      printf("*** before re-running configure\n");
			return 1;
    } 
}
],, no_mino=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_mino" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$MINO_CONFIG" = "no" ; then
       echo "*** The mino-config script installed by Mino could not be found"
       echo "*** If Mino was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the MINO_CONFIG environment variable to the"
       echo "*** full path to mino-config."
     else
       if test -f conf.minotest ; then
        :
       else
          echo "*** Could not run Mino test program, checking why..."
          CFLAGS="$CFLAGS $MINO_CFLAGS"
          LIBS="$LIBS $MINO_LIBS"
          AC_TRY_LINK([
#include <mino.h>
#include <stdio.h>
],      [ return ((mino_major_version) || (mino_minor_version) || (mino_micro_version)); ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding Mino or finding the wrong"
          echo "*** version of Mino. If it is not finding Mino, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
          echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"
          echo "***"
          echo "*** If you have a RedHat 5.0 system, you should remove the Mino package that"
          echo "*** came with the system with the command"
          echo "***"
          echo "***    rpm --erase --nodeps mino mino-devel" ],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means Mino was incorrectly installed"
          echo "*** or that you have moved Mino since it was installed. In the latter case, you"
          echo "*** may want to edit the mino-config script: $MINO_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     MINO_CFLAGS=""
     MINO_LIBS=""
     ifelse([$3], , :, [$3])
		 AC_LANG_RESTORE
  fi
  AC_SUBST(MINO_CFLAGS)
  AC_SUBST(MINO_LIBS)
  rm -f conf.minotest
])
