/**
 * @file generate.c Base generation functions
 * 
 * $Id: generate.c,v 1.9 2001/12/31 09:26:54 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

void
updateGenerationStatus(GenerationState *state, const char *baseMessage,
					   const char *specific)
{
	char *msg = (char *)malloc((strlen(baseMessage) + strlen(specific) + 1) *
							   sizeof(char));

	strcpy(msg, baseMessage);
	strcat(msg, specific);

	dialogUpdateGauge(state->gauge, state->currentPercent, msg);
	state->currentPercent += state->percentIncrement;

	free(msg);
}

void
generate(ProjectInfo *project)
{
	GenerationState *state;
	
	state = (GenerationState *)malloc(sizeof(GenerationState));

	/*
	 * total: 13
	 * bin: 2
	 * lib: 4
	 */
	
	/* Figure out total amount of steps */
	state->totalSteps = 13;

	if (project->target == TARGET_EXECUTABLE) state->totalSteps += 2;
	if (project->target == TARGET_LIBRARY)    state->totalSteps += 4;
	
	state->percentIncrement = 100 / state->totalSteps;
	state->currentPercent = 0;
	
	state->gauge = newDialogGauge("Generating Source Code Tree",
								  "Beginning generation...");

	generateDirectories(project, state);
	generateDocumentation(project, state);
	generateAutogen(project, state);
	generateConfigureIn(project, state);
	generateAcconfig(project, state);
	generateMacros(project, state);
	generateMakefiles(project, state);
	generateLibConfig(project, state);
	generateM4(project, state);
	generateRpmSpec(project, state);
	
	dialogUpdateGauge(state->gauge, 100, "Done!");

	freeDialogGauge(state->gauge);

	free(state);
}

