/**
 * @file generate_configin.c configure.in generation
 * 
 * $Id: generate_configin.c,v 1.10 2002/07/07 21:47:46 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

void
splitVersion(char *version, char *ret[3])
{
	char buffer[128];
	char *c, *d;
	int i = 0;

	d = buffer;
	
	for (c = version; *c != '\0'; c++)
	{
		if (*c == '.')
		{
			*d = '\0';
			
			ret[i++] = strdup(buffer);

			if (i == 3)
				return;
			
			d = buffer;
		}
		else
		{
			*d++ = *c;
		}
	}
}

void
configComment(FILE *f, char *text)
{
	fprintf(f, "\ndnl ################################"
	                 "################################\n");
	fprintf(f, "dnl # %s\n", text);
	fprintf(f, "dnl ################################"
	               "################################\n");
}

void
generateConfigureIn(ProjectInfo *project, GenerationState *state)
{
	char *filename;
	char *ucName;
	char *temp;
	char *version[3];
	FILE *f;

	Package *pkg;
	Check *check;
	
	updateGenerationStatus(state, "Creating:\n\t", "configure.in");
	
	filename = (char *)malloc((strlen(project->dir) + 14) * sizeof(char));
	
	strcpy(filename, project->dir);
	strcat(filename, "/configure.in");

	if ((f = fopen(filename, "w")) == NULL)
	{
		fprintf(stderr, "Could not write to %s\n", filename);
		exit(1);
	}

	free(filename);

	temp = stringToUpper(project->name);
	ucName = fixProjectName(temp);
	free(temp);

	splitVersion(project->version, version);
		
	fprintf(f, "dnl Process this file with autoconf to create configure.\n\n");
	
	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
		fprintf(f, "AC_INIT(%s-config.h.in)\n", project->name);
	else
		fprintf(f, "AC_INIT(config.h.in)\n");

	configComment(f, "Version information");
	fprintf(f, "%s_MAJOR_VERSION=%s\n", ucName, version[0]);
	fprintf(f, "%s_MINOR_VERSION=%s\n", ucName, version[1]);
	fprintf(f, "%s_MICRO_VERSION=%s\n\n", ucName, version[2]);

	fprintf(f, "%s_VERSION=$%s_MAJOR_VERSION.$%s_MINOR_VERSION."
			   "$%s_MICRO_VERSION\n", ucName, ucName, ucName, ucName);
	fprintf(f, "AC_DEFINE_UNQUOTED(%s_MAJOR_VERSION, $%s_MAJOR_VERSION)\n",
			ucName, ucName);
	fprintf(f, "AC_DEFINE_UNQUOTED(%s_MINOR_VERSION, $%s_MINOR_VERSION)\n",
			ucName, ucName);
	fprintf(f, "AC_DEFINE_UNQUOTED(%s_MICRO_VERSION, $%s_MICRO_VERSION)\n",
			ucName, ucName);
	fprintf(f, "AC_DEFINE_UNQUOTED(%s_VERSION, \"$%s_VERSION\")\n",
			ucName, ucName);
	
	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		configComment(f, "libtool versioning");
		fprintf(f, "dnl #\n");
		fprintf(f, "dnl # +1 :  ? : +1   == new interface that does not"
				   " break old one\n");
		fprintf(f, "dnl # +1 :  ? :  0   == new interface that breaks old"
				   " one\n");
		fprintf(f, "dnl #  ? :  ? :  0   == no new interfaces, but breaks"
				   " apps\n");
		fprintf(f, "dnl #  ? : +1 :  ?   == just some internal changes,"
				   " nothing breaks but\n");
		fprintf(f, "dnl #                   might work better\n");
		fprintf(f, "dnl # CURRENT : REVISION : AGE\n");
		fprintf(f, "LT_CURRENT=0\n");
		fprintf(f, "LT_REVISION=0\n");
		fprintf(f, "LT_AGE=0\n\n");
		
		fprintf(f, "LT_RELEASE=$%s_MAJOR_VERSION.$%s_MINOR_VERSION."
				   "$%s_MICRO_VERSION\n\n", ucName, ucName, ucName);
		
		fprintf(f, "AC_SUBST(LT_RELEASE)\n");
		fprintf(f, "AC_SUBST(LT_CURRENT)\n");
		fprintf(f, "AC_SUBST(LT_REVISION)\n");
		fprintf(f, "AC_SUBST(LT_AGE)\n");
	}

	configComment(f, "Initialize automake");
	fprintf(f, "VERSION=$%s_VERSION\n", ucName);
	fprintf(f, "PACKAGE=%s\n\n", project->name);

	fprintf(f, "AM_INIT_AUTOMAKE($PACKAGE, $VERSION)\n");

	configComment(f, "Specify a header configuration file");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
		fprintf(f, "AM_CONFIG_HEADER(%s-config.h)\n", project->name);
	else
		fprintf(f, "AM_CONFIG_HEADER(config.h)\n");
	

	configComment(f, "Check for some standard stuff.");
	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "AC_LIBTOOL_DLOPEN\n");
		fprintf(f, "AM_PROG_LIBTOOL\n");
	}

	if (project->languages->c)
		fprintf(f, "AC_PROG_CC\n");

	if (project->languages->cpp)
		fprintf(f, "AC_PROG_CXX\n");

	if (project->languages->fortran)
	{
		fprintf(f, "AC_PROG_F77\n");
		fprintf(f, "AC_F77_LIBRARY_LDFLAGS\n");
	}

	if (project->languages->yacc)
		fprintf(f, "AC_PROG_YACC\n");

	if (project->languages->lex)
	{
		fprintf(f, "AC_PROG_LEX\n");
		fprintf(f, "AC_DECL_YYTEXT\n");
	}

	if (project->languages->awk)
		fprintf(f, "AC_PROG_AWK\n");
	
	fprintf(f, "AC_ARG_PROGRAM\n");
	fprintf(f, "AC_PROG_CPP\n");
	fprintf(f, "AC_PROG_MAKE_SET\n");
	fprintf(f, "AC_PROG_LN_S\n");
	fprintf(f, "AC_PROG_INSTALL\n\n");
	
	fprintf(f, "AC_C_CONST\n");
	fprintf(f, "AC_TYPE_SIZE_T\n");
	fprintf(f, "AC_EXEEXT\n");
	fprintf(f, "AC_HEADER_STDC\n");


	/*
	 * Okay now, let's output all the wonderful package stuff!
	 */
	for (pkg = project->packages; pkg != NULL; pkg = pkg->next)
	{
		char *temp;
		
		temp = (char *)malloc((strlen(pkg->name) + 12) * sizeof(char));

		strcpy(temp, "Checks for ");
		strcat(temp, pkg->name);
		
		configComment(f, temp);

		free(temp);

		for (check = pkg->checks; check != NULL; check = check->next)
		{
			if (check->macro != NULL)
				fprintf(f, "%s\n", check->macro);
		}
	}

	configComment(f, "Some substs and stuff");
	fprintf(f, "AC_SUBST(CFLAGS)\n");
	fprintf(f, "AC_SUBST(CPPFLAGS)\n");
	fprintf(f, "AC_SUBST(LDFLAGS)\n");

	configComment(f, "Output the Makefiles");
	fprintf(f, "AC_OUTPUT([\n");
	fprintf(f, "%s.spec\n", project->name);

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "%s-config\n", project->name);
		fprintf(f, "%s.m4\n", project->name);
	}

	fprintf(f, "Makefile\n");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "lib%s/Makefile\n", project->name);
	}

	if (project->target == TARGET_BOTH || project->target == TARGET_EXECUTABLE)
	{
		fprintf(f, "src/Makefile\n");
	}

	fprintf(f, "])\n");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "\nchmod +x %s-config\n", project->name);
	}
	
	fclose(f);

	fprintf(stderr, "[%s]\n", project->name);

#if 0
	for (i = 0; i < 3; i++)
		free(version[i]);
#endif

	fprintf(stderr, "[%s]\n", project->name);

	free(ucName);

	fprintf(stderr, "[%s]\n", project->name);
}

