/**
 * @file generateMakefiles.c Makefile.am generation
 * 
 * $Id: generate_makefiles.c,v 1.7 2001/12/31 09:26:55 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

void
generateMakefiles(ProjectInfo *project, GenerationState *state)
{
	Package *pkg;
	Check   *check;
	char    *filename;
	char    *newName = fixProjectName(project->name);
	FILE    *f;

	/*
	 * Makefile.am
	 */
	updateGenerationStatus(state, "Creating:\n\t", "Makefile.am");

	filename = (char *)malloc((strlen(project->dir) + 13) * sizeof(char));
	strcpy(filename, project->dir);
	strcat(filename, "/Makefile.am");

	if ((f = fopen(filename, "w")) == NULL)
	{
		fprintf(stderr, "Could not write to %s\n", filename);
		exit(1);
	}

	free(filename);
	
	fprintf(f, "# $" "Id" "$\n\n");
	fprintf(f, "SUBDIRS =");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, " lib%s", project->name);
	}
	
	if (project->target == TARGET_BOTH || project->target == TARGET_EXECUTABLE)
		fprintf(f, " src");

	fprintf(f, "\n\n");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "bin_SCRIPTS = %s-config\n\n", project->name);
	}

	fprintf(f, "EXTRA_DIST = \\\n");
	fprintf(f, "\tAUTHORS NEWS INSTALL COPYING \\\n");
	fprintf(f, "\tChangeLog \\\n");
	fprintf(f, "\tautogen.sh \\\n");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "\t%s.m4.in \\\n", project->name);
	}

	fprintf(f, "\t%s.spec.in\n\n", project->name);

	
	fprintf(f, "dist-hook: %s.spec\n", project->name);
	fprintf(f, "\tcp %s.spec $(distdir)\n\n", project->name);

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "m4datadir = $(datadir)/aclocal\n");
		fprintf(f, "m4data_DATA = %s.m4\n\n", project->name);
		
		fprintf(f, "$(OBJECTS): libtool\n");
		fprintf(f, "libtool: $(LIBTOOL_DEPS)\n");
		fprintf(f, "\t$(SHELL) ./config.status --recheck\n\n");
	}

	fprintf(f, "release:\n");
	fprintf(f, "\t$(MAKE) dist distdir=$(PACKAGE)`date + \"%%y%%m%%d\"`\n");
	
	fclose(f);

	/*
	 * lib/makefile.am
	 */
	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		filename = (char *)malloc((strlen(project->name) + 16) * sizeof(char));
		
		sprintf(filename, "lib%s/Makefile.am", project->name);
		updateGenerationStatus(state, "Creating:\n\t", filename);
		
		free(filename);
		
		filename = (char *)malloc((strlen(project->dir) +
								   strlen(project->name) + 17) * sizeof(char));
		sprintf(filename, "%s/lib%s/Makefile.am", project->dir, project->name);

		if ((f = fopen(filename, "w")) == NULL)
		{
			fprintf(stderr, "Could not write to %s\n", filename);
			exit(1);
		}

		free(filename);
	
		fprintf(f, "# $" "Id" "$\n\n");
		fprintf(f, "%sincludedir = $(includedir)/lib%s\n\n",
				project->name, project->name);
		
		fprintf(f, "%sinclude_HEADERS = \n\n", newName);
		fprintf(f, "lib_LTLIBRARIES = lib%s.la\n\n", project->name);
		fprintf(f, "lib%s_la_SOURCES = \n\n", newName);
		fprintf(f, "lib%s_la_LIBADD = ", newName);

		for (pkg = project->packages; pkg != NULL; pkg = pkg->next)
		{
			for (check = pkg->checks; check != NULL; check = check->next)
			{
				if (check->libs != NULL)
				{
					fprintf(f, "\\\n");
					fprintf(f, "\t$(%s) ", check->libs);
				}
			}
		}

		fprintf(f, "\n\n");

		fprintf(f, "lib%s_la_LDFLAGS = \\\n", newName);
		fprintf(f, "\t-version-info"
				   " $(LT_CURRENT):$(LT_REVISION):$(LT_AGE)\n\n");
		
		fprintf(f, "INCLUDES = \\\n");
		fprintf(f, "\t-I$(top_srcdir) \\\n");
		fprintf(f, "\t-I$(includedir) \\\n");
		fprintf(f, "\t-I$(prefix)/include ");

		for (pkg = project->packages; pkg != NULL; pkg = pkg->next)
		{
			for (check = pkg->checks; check != NULL; check = check->next)
			{
				if (check->cflags != NULL)
				{
					fprintf(f, "\\\n");
					fprintf(f, "\t$(%s) ", check->cflags);
				}
			}
		}

		fprintf(f, "\n");

		fclose(f);
	}
	
	/*
	 * src/Makefile.am
	 */
	if (project->target == TARGET_BOTH || project->target == TARGET_EXECUTABLE)
	{
		updateGenerationStatus(state, "Creating:\n\t", "src/Makefile.am");
		
		filename = (char *)malloc((strlen(project->dir) + 17) * sizeof(char));
		sprintf(filename, "%s/src/Makefile.am", project->dir);

		if ((f = fopen(filename, "w")) == NULL)
		{
			fprintf(stderr, "Could not write to %s\n", filename);
			exit(1);
		}

		free(filename);

		fprintf(f, "# $" "Id" "$\n\n");
		fprintf(f, "bin_PROGRAMS = %s\n\n", project->name);
		fprintf(f, "%s_SOURCES = \n\n", newName);
		fprintf(f, "%s_LDADD = ", newName);

		if (project->target == TARGET_BOTH)
		{
			fprintf(f, "\\\n");
			fprintf(f, "\t$(top_builddir)/lib%s/lib%s.la ", project->name,
					project->name);
		}

		for (pkg = project->packages; pkg != NULL; pkg = pkg->next)
		{
			for (check = pkg->checks; check != NULL; check = check->next)
			{
				if (check->libs != NULL)
				{
					fprintf(f, "\\\n");
					fprintf(f, "\t$(%s) ", check->libs);
				}
			}
		}

		fprintf(f, "\n\n");

		fprintf(f, "INCLUDES = \\\n");
		fprintf(f, "\t-I$(top_srcdir) \\\n");
		fprintf(f, "\t-I$(includedir) \\\n");
		fprintf(f, "\t-I$(prefix)/include ");
		
		for (pkg = project->packages; pkg != NULL; pkg = pkg->next)
		{
			for (check = pkg->checks; check != NULL; check = check->next)
			{
				if (check->cflags != NULL)
				{
					fprintf(f, "\\\n");
					fprintf(f, "\t$(%s) ", check->cflags);
				}
			}
		}

		fprintf(f, "\n");

		fclose(f);
	}
	
	fclose(f);
}

