/**
 * @file main.c Main function
 * 
 * $Id: main.c,v 1.7 2002/10/21 19:15:40 mhearn Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifdef ENABLE_LIBPREFIXDB
# include <prefixdb.h>
# include <limits.h>
#endif /* ENABLE_LIBPREFIXDB */

#include "genst.h"

char *data_dir = NULL;

int
main(int argc, char **argv)
{
	ProjectInfo *project = newProjectInfo();

	#ifdef ENABLE_LIBPREFIXDB
		PrefixDB *db;

		db = pdbAutoInit("@gnupdate.org");
		data_dir = calloc(PATH_MAX + 1, 1);
		snprintf(data_dir, PATH_MAX, "%s/%s", pdbGetShare(db), PDB_APPNAME);
	#else
		data_dir = DATA_DIR;
	#endif /* ENABLE_LIBPREFIXDB */

	setCurrentProject(project);
	
	/**********************************************************************
	 * Display a welcome message.
	 **********************************************************************/
	dialogMsgBox("Hello, and welcome to genst! I'll be asking you a few\n"
				  "simple questions before I build your source tree.\n\n"
				  "Are you ready to begin?");
	
	requestProjectName(project);
	requestProjectVersion(project);
	requestProjectLicense(project);
	requestProjectSummary(project);
	requestProjectGroup(project);
	requestProjectTarget(project);
	requestProjectLanguage(project);
	requestProjectDirectory(project);
	requestPackages(project);

	dialogMsgBox("The generation is complete. Please fill out the\n"
				  "missing information in your spec file as soon\n"
				  "as possible.\n");

	freeProjectInfo(project);
	
#ifdef WITH_LEAKBUG
	lbDumpLeaks();
#endif
	
	return 0;
}
