/**
 * @file xml.c XML utility functions
 * 
 * $Id: xml.c,v 1.6 2002/07/07 21:47:46 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

#include <stdarg.h>
#include <libxml/parser.h>
#include <libxml/parserInternals.h>

int
parseXml(xmlSAXHandlerPtr sax, void *userData, const char *filename)
{
	int ret = 0;
	xmlParserCtxtPtr ctxt;

	ctxt = xmlCreateFileParserCtxt(filename);
	
	if (ctxt == NULL)
		return -1;

	ctxt->userData = userData;
	ctxt->sax      = sax;
	
	xmlParseDocument(ctxt);

	ret = (ctxt->wellFormed ? 0 : -1);

	if (sax != NULL)
		ctxt->sax = NULL;

	xmlFreeParserCtxt(ctxt);

	return ret;
}

void
xmlWarning(void *userData, const char *msg, ...)
{
	va_list args;

	va_start(args, msg);
	
	printf("Warning: ");

	while (*msg)
	{
		printf("%s", va_arg(args, char *));
	}

	printf("\n");

	va_end(args);
}

void
xmlError(void *userData, const char *msg, ...)
{
	va_list args;

	va_start(args, msg);
	
	printf("Error: ");

	while (*msg)
	{
		printf("%s", va_arg(args, char *));
	}

	printf("\n");

	va_end(args);
}

void
xmlFatalError(void *userData, const char *msg, ...)
{
	va_list args;

	va_start(args, msg);
	
	printf("Fatal Error: ");

	while (*msg)
	{
		printf("%s", va_arg(args, char *));
	}

	printf("\n");

	va_end(args);

	exit(1);
}

